# Copyright (c) 2012 OpenStack, LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from numbers import Number
import os
import re
import shutil
import six
import socket
import time
import unittest

import pexpect

from proboscis import test
from proboscis.asserts import assert_raises
from proboscis.decorators import expect_exception
from proboscis.decorators import time_out

from trove.tests.config import CONFIG
from trove.common.utils import poll_until
from trove.tests.util import process
from trove.common.utils import import_class
from tests import initialize


WHITE_BOX = CONFIG.white_box
VOLUMES_DRIVER = "trove.volumes.driver"

if WHITE_BOX:
    # TODO(tim.simpson): Restore this once white box functionality can be
    #                    added back to this test module.
    pass
    # from nova import context
    # from nova import exception
    # from nova import flags
    # from nova import utils
    # from trove import exception as trove_exception
    # from trove.utils import poll_until
    # from trove import volume
    # from trove.tests.volume import driver as test_driver

    # FLAGS = flags.FLAGS


UUID_PATTERN = re.compile('^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-'
                          '[0-9a-f]{4}-[0-9a-f]{12}$')

HUGE_VOLUME = 5000


def is_uuid(text):
    return UUID_PATTERN.search(text) is not None


class StoryDetails(object):

    def __init__(self):
        self.api = volume.API()
        self.client = volume.Client()
        self.context = context.get_admin_context()
        self.device_path = None
        self.volume_desc = None
        self.volume_id = None
        self.volume_name = None
        self.volume = None
        self.host = socket.gethostname()
        self.original_uuid = None
        self.original_device_info = None
        self.resize_volume_size = 2

    def get_volume(self):
        return self.api.get(self.context, self.volume_id)

    @property
    def mount_point(self):
        return "%s/%s" % (LOCAL_MOUNT_PATH, self.volume_id)

    @property
    def test_mount_file_path(self):
        return "%s/test.txt" % self.mount_point


story = None
storyFail = None

LOCAL_MOUNT_PATH = "/testsmnt"


class VolumeTest(unittest.TestCase):
    """This test tells the story of a volume, from cradle to grave."""

    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)

    def setUp(self):
        global story, storyFail
        self.story = story
        self.storyFail = storyFail

    def assert_volume_as_expected(self, volume):
        self.assertIsInstance(volume["id"], Number)
        self.assertEqual(self.story.volume_name, volume["display_name"])
        self.assertEqual(self.story.volume_desc, volume["display_description"])
        self.assertEqual(1, volume["size"])
        self.assertEqual(self.story.context.user_id, volume["user_id"])
        self.assertEqual(self.story.context.project_id, volume["project_id"])


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[initialize.start_volume])
class SetUp(VolumeTest):

    def test_05_create_story(self):
        """Creating 'story' vars used by the rest of these tests."""
        global story, storyFail
        story = StoryDetails()
        storyFail = StoryDetails()

    @time_out(60)
    def test_10_wait_for_topics(self):
        """Wait until the volume topic is up before proceeding."""
        topics = ["volume"]
        from tests.util.topics import hosts_up
        while not all(hosts_up(topic) for topic in topics):
            pass

    def test_20_refresh_local_folders(self):
        """Delete the local folders used as mount locations if they exist."""
        if os.path.exists(LOCAL_MOUNT_PATH):
            #TODO(rnirmal): Also need to remove any existing mounts.
            shutil.rmtree(LOCAL_MOUNT_PATH)
        os.mkdir(LOCAL_MOUNT_PATH)
        # Give some time for the services to startup
        time.sleep(10)

    @time_out(60)
    def test_30_mgmt_volume_check(self):
        """Get the volume information from the mgmt API"""
        story_context = self.story.context
        device_info = self.story.api.get_storage_device_info(story_context)
        print("device_info : %r" % device_info)
        self.assertNotEqual(device_info, None,
            "the storage device information should exist")
        self.story.original_device_info = device_info

    @time_out(60)
    def test_31_mgmt_volume_info(self):
        """Check the available space against the mgmt API info."""
        story_context = self.story.context
        device_info = self.story.api.get_storage_device_info(story_context)
        print("device_info : %r" % device_info)
        info = {'spaceTotal': device_info['raw_total'],
                'spaceAvail': device_info['raw_avail']}
        self._assert_available_space(info)

    def _assert_available_space(self, device_info, fail=False):
        """
        Give the SAN device_info(fake or not) and get the asserts for free
        """
        print("DEVICE_INFO on SAN : %r" % device_info)
        # Calculate the GBs; Divide by 2 for the FLAGS.san_network_raid_factor
        gbs = 1.0 / 1024 / 1024 / 1024 / 2
        total = int(device_info['spaceTotal']) * gbs
        free = int(device_info['spaceAvail']) * gbs
        used = total - free
        usable = total * (FLAGS.san_max_provision_percent * 0.01)
        real_free = float(int(usable - used))

        print("total : %r" % total)
        print("free : %r" % free)
        print("used : %r" % used)
        print("usable : %r" % usable)
        print("real_free : %r" % real_free)

        check_space = self.story.api.check_for_available_space
        self.assertFalse(check_space(self.story.context, HUGE_VOLUME))
        self.assertFalse(check_space(self.story.context, real_free + 1))

        if fail:
            self.assertFalse(check_space(self.story.context, real_free))
            self.assertFalse(check_space(self.story.context, real_free - 1))
            self.assertFalse(check_space(self.story.context, 1))
        else:
            self.assertTrue(check_space(self.story.context, real_free))
            self.assertTrue(check_space(self.story.context, real_free - 1))
            self.assertTrue(check_space(self.story.context, 1))


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[SetUp])
class AddVolumeFailure(VolumeTest):

    @time_out(60)
    def test_add(self):
        """
        Make call to FAIL a prov. volume and assert the return value is a
        FAILURE.
        """
        self.assertIsNone(self.storyFail.volume_id)
        name = "TestVolume"
        desc = "A volume that was created for testing."
        self.storyFail.volume_name = name
        self.storyFail.volume_desc = desc
        volume = self.storyFail.api.create(self.storyFail.context,
                                           size=HUGE_VOLUME,
                                           snapshot_id=None, name=name,
                                           description=desc)
        self.assertEqual(HUGE_VOLUME, volume["size"])
        self.assertEqual("creating", volume["status"])
        self.assertEqual("detached", volume["attach_status"])
        self.storyFail.volume = volume
        self.storyFail.volume_id = volume["id"]


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[AddVolumeFailure])
class AfterVolumeFailureIsAdded(VolumeTest):
    """Check that the volume can be retrieved via the API, and setup.

    All we want to see returned is a list-like with an initial string.

    """

    @time_out(120)
    def test_api_get(self):
        """Wait until the volume is a FAILURE."""
        volume = poll_until(lambda: self.storyFail.get_volume(),
                            lambda volume: volume["status"] != "creating")
        self.assertEqual("error", volume["status"])
        self.assertEqual("detached", volume["attach_status"])

    @time_out(60)
    def test_mgmt_volume_check(self):
        """Get the volume information from the mgmt API"""
        info = self.story.api.get_storage_device_info(self.story.context)
        print("device_info : %r" % info)
        self.assertNotEqual(info, None,
                            "the storage device information should exist")
        self.assertEqual(self.story.original_device_info['raw_total'],
                         info['raw_total'])
        self.assertEqual(self.story.original_device_info['raw_avail'],
                         info['raw_avail'])


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[SetUp])
class AddVolume(VolumeTest):

    @time_out(60)
    def test_add(self):
        """Make call to prov. a volume and assert the return value is OK."""
        self.assertIsNone(self.story.volume_id)
        name = "TestVolume"
        desc = "A volume that was created for testing."
        self.story.volume_name = name
        self.story.volume_desc = desc
        volume = self.story.api.create(self.story.context, size=1,
                                       snapshot_id=None, name=name,
                                       description=desc)
        self.assert_volume_as_expected(volume)
        self.assertEqual("creating", volume["status"])
        self.assertEqual("detached", volume["attach_status"])
        self.story.volume = volume
        self.story.volume_id = volume["id"]


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[AddVolume])
class AfterVolumeIsAdded(VolumeTest):
    """Check that the volume can be retrieved via the API, and setup.

    All we want to see returned is a list-like with an initial string.

    """

    @time_out(120)
    def test_api_get(self):
        """Wait until the volume is finished provisioning."""
        volume = poll_until(lambda: self.story.get_volume(),
                            lambda volume: volume["status"] != "creating")
        self.assertEqual("available", volume["status"])
        self.assert_volume_as_expected(volume)
        self.assertEqual("detached", volume["attach_status"])

    @time_out(60)
    def test_mgmt_volume_check(self):
        """Get the volume information from the mgmt API"""
        print("self.story.original_device_info : %r" %
              self.story.original_device_info)
        info = self.story.api.get_storage_device_info(self.story.context)
        print("device_info : %r" % info)
        self.assertNotEqual(info, None,
                            "the storage device information should exist")
        self.assertEqual(self.story.original_device_info['raw_total'],
                         info['raw_total'])
        volume_size = int(self.story.volume['size']) * (1024 ** 3) * 2
        print("volume_size: %r" % volume_size)
        print("self.story.volume['size']: %r" % self.story.volume['size'])
        avail = int(self.story.original_device_info['raw_avail']) - volume_size
        print("avail space: %r" % avail)
        self.assertEqual(int(info['raw_avail']), avail)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[AfterVolumeIsAdded])
class SetupVolume(VolumeTest):

    @time_out(60)
    def test_assign_volume(self):
        """Tell the volume it belongs to this host node."""
        #TODO(tim.simpson) If this is important, could we add a test to
        #                  make sure some kind of exception is thrown if it
        #                  isn't added to certain drivers?
        self.assertNotEqual(None, self.story.volume_id)
        self.story.api.assign_to_compute(self.story.context,
                                         self.story.volume_id,
                                         self.story.host)

    @time_out(60)
    def test_setup_volume(self):
        """Set up the volume on this host. AKA discovery."""
        self.assertNotEqual(None, self.story.volume_id)
        device = self.story.client._setup_volume(self.story.context,
                                                self.story.volume_id,
                                                self.story.host)
        if not isinstance(device, six.string_types):
            self.fail("Expected device to be a string, but instead it was " +
                      str(type(device)) + ".")
        self.story.device_path = device


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[SetupVolume])
class FormatVolume(VolumeTest):

    @expect_exception(IOError)
    @time_out(60)
    def test_10_should_raise_IOError_if_format_fails(self):
        """

        Tests that if the driver's _format method fails, its
        public format method will perform an assertion properly, discover
        it failed, and raise an exception.

        """

        volume_driver_cls = import_class(FLAGS.volume_driver)

        class BadFormatter(volume_driver_cls):

            def _format(self, device_path):
                pass

        bad_client = volume.Client(volume_driver=BadFormatter())
        bad_client._format(self.story.device_path)

    @time_out(60)
    def test_20_format(self):
        self.assertNotEqual(None, self.story.device_path)
        self.story.client._format(self.story.device_path)

    def test_30_check_options(self):
        cmd = ("sudo dumpe2fs -h %s 2> /dev/null | "
               "awk -F ':' '{ if($1 == \"Reserved block count\") "
               "{ rescnt=$2 } } { if($1 == \"Block count\") "
               "{ blkcnt=$2 } } END { print (rescnt/blkcnt)*100 }'")
        cmd = cmd % self.story.device_path
        out, err = process(cmd)
        self.assertEqual(float(5), round(float(out)), msg=out)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[FormatVolume])
class MountVolume(VolumeTest):

    @time_out(60)
    def test_mount(self):
        self.story.client._mount(self.story.device_path,
                                 self.story.mount_point)
        with open(self.story.test_mount_file_path, 'w') as file:
            file.write("Yep, it's mounted alright.")
        self.assertTrue(os.path.exists(self.story.test_mount_file_path))

    def test_mount_options(self):
        cmd = "mount -l | awk '/%s.*noatime/ { print $1 }'"
        cmd %= LOCAL_MOUNT_PATH.replace('/', '')
        out, err = process(cmd)
        self.assertEqual(os.path.realpath(self.story.device_path), out.strip(),
                         msg=out)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[MountVolume])
class ResizeVolume(VolumeTest):

    @time_out(300)
    def test_resize(self):
        self.story.api.resize(self.story.context, self.story.volume_id,
                              self.story.resize_volume_size)

        volume = poll_until(lambda: self.story.get_volume(),
                            lambda volume: volume["status"] == "resized")
        self.assertEqual("resized", volume["status"])
        self.assertEqual("attached", volume["attach_status"])
        self.assertEqual(self.story.resize_volume_size, volume['size'])

    @time_out(300)
    def test_resizefs_rescan(self):
        self.story.client.resize_fs(self.story.context,
                                    self.story.volume_id)
        expected = "trove.tests.volume.driver.ISCSITestDriver"
        if FLAGS.volume_driver is expected:
            size = self.story.resize_volume_size * \
                   test_driver.TESTS_VOLUME_SIZE_MULTIPLIER * 1024 * 1024
        else:
            size = self.story.resize_volume_size * 1024 * 1024
        out, err = process('sudo blockdev --getsize64 %s' %
                           os.path.realpath(self.story.device_path))
        if int(out) < (size * 0.8):
            self.fail("Size %s is not more or less %s" % (out, size))

        # Reset the volume status to available
        self.story.api.update(self.story.context, self.story.volume_id,
                              {'status': 'available'})


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[MountVolume])
class UnmountVolume(VolumeTest):

    @time_out(60)
    def test_unmount(self):
        self.story.client._unmount(self.story.mount_point)
        child = pexpect.spawn("sudo mount %s" % self.story.mount_point)
        child.expect("mount: can't find %s in" % self.story.mount_point)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[UnmountVolume])
class GrabUuid(VolumeTest):

    @time_out(60)
    def test_uuid_must_match_pattern(self):
        """UUID must be hex chars in the form 8-4-4-4-12."""
        client = self.story.client  # volume.Client()
        device_path = self.story.device_path  # '/dev/sda5'
        uuid = client.get_uuid(device_path)
        self.story.original_uuid = uuid
        self.assertTrue(is_uuid(uuid), "uuid must match regex")

    @time_out(60)
    def test_get_invalid_uuid(self):
        """DevicePathInvalidForUuid is raised if device_path is wrong."""
        client = self.story.client
        device_path = "gdfjghsfjkhggrsyiyerreygghdsghsdfjhf"
        self.assertRaises(trove_exception.DevicePathInvalidForUuid,
                          client.get_uuid, device_path)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[GrabUuid])
class RemoveVolume(VolumeTest):

    @time_out(60)
    def test_remove(self):
        self.story.client.remove_volume(self.story.context,
                                 self.story.volume_id,
                                 self.story.host)
        self.assertRaises(Exception,
                          self.story.client._format, self.story.device_path)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[GrabUuid])
class Initialize(VolumeTest):

    @time_out(300)
    def test_10_initialize_will_format(self):
        """initialize will setup, format, and store the UUID of a volume"""
        self.assertTrue(self.story.get_volume()['uuid'] is None)
        self.story.client.initialize(self.story.context, self.story.volume_id,
                                     self.story.host)
        volume = self.story.get_volume()
        self.assertTrue(is_uuid(volume['uuid']), "uuid must match regex")
        self.assertNotEqual(self.story.original_uuid, volume['uuid'],
                            "Validate our assumption that the volume UUID "
                            "will change when the volume is formatted.")
        self.story.client.remove_volume(self.story.context,
                                        self.story.volume_id,
                                        self.story.host)

    @time_out(60)
    def test_20_initialize_the_second_time_will_not_format(self):
        """If initialize is called but a UUID exists, it should not format."""
        old_uuid = self.story.get_volume()['uuid']
        self.assertTrue(old_uuid is not None)

        class VolumeClientNoFmt(volume.Client):

            def _format(self, device_path):
                raise RuntimeError("_format should not be called!")

        no_fmt_client = VolumeClientNoFmt()
        no_fmt_client.initialize(self.story.context, self.story.volume_id,
                                 self.story.host)
        self.assertEqual(old_uuid, self.story.get_volume()['uuid'],
                         "UUID should be the same as no formatting occurred.")
        self.story.client.remove_volume(self.story.context,
                                        self.story.volume_id,
                                        self.story.host)

    def test_30_check_device_exists(self):
        assert_raises(exception.InvalidDevicePath, self.story.client._format,
                      self.story.device_path)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[Initialize])
class DeleteVolume(VolumeTest):

    @time_out(60)
    def test_delete(self):
        self.story.api.delete(self.story.context, self.story.volume_id)


@test(groups=[VOLUMES_DRIVER], depends_on_classes=[DeleteVolume])
class ConfirmMissing(VolumeTest):

    @time_out(60)
    def test_discover_should_fail(self):
        try:
            self.story.client.driver.discover_volume(self.story.context,
                                                     self.story.volume)
            self.fail("Expecting an error but did not get one.")
        except exception.Error:
            pass
        except trove_exception.ISCSITargetNotDiscoverable:
            pass

    @time_out(60)
    def test_get_missing_volume(self):
        try:
            volume = poll_until(lambda: self.story.api.get(self.story.context,
                                                        self.story.volume_id),
                                lambda volume: volume["status"] != "deleted")
            self.assertEqual(volume["deleted"], False)
        except exception.VolumeNotFound:
            pass



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
This test recreates an issue we had with eventlet. In the logs, we'd see that
the JSON response was malformed; instead of JSON, it contained the following
string:
Second simultaneous read on fileno 5 detected.  Unless you really know what
you're doing, make sure that only one greenthread can read any particular
socket.  Consider using a pools.Pool. If you do know what you're doing and want
to disable this error, call
eventlet.debug.hub_multiple_reader_prevention(False)

It is perhaps the most helpful error message ever created.

The root issue was that a subclass of httplib2.Http was created at program
started and used in all threads.

Using the old (broken) RsDNS client code this test recreates the greatest error
message ever.
"""

try:
    import eventlet
    CAN_USE_EVENTLET = True
except ImportError:
    CAN_USE_EVENTLET = False
import uuid

from proboscis import before_class
from proboscis import test
from proboscis.asserts import assert_true

from trove.tests.config import CONFIG

WHITE_BOX = CONFIG.white_box
RUN_DNS = CONFIG.values.get("trove_dns_support", False)


if CONFIG.white_box:
    from trove.dns.rsdns.driver import RsDnsInstanceEntryFactory
    from nova import flags
    from nova import utils
    FLAGS = flags.FLAGS


@test(groups=["rsdns.eventlet"], enabled=CAN_USE_EVENTLET)
class RsdnsEventletTests(object):
    """Makes sure the RSDNS client can be used from multiple green threads."""

    def assert_record_created(self, index):
        msg = "Record %d wasn't created!" % index
        assert_true(index in self.new_records, msg)

    @before_class(enabled=WHITE_BOX and RUN_DNS)
    def create_driver(self):
        """Creates the DNS Driver used in subsequent tests."""
        self.driver = utils.import_object(FLAGS.dns_driver)
        self.entry_factory = RsDnsInstanceEntryFactory()
        self.test_uuid = uuid.uuid4().hex
        self.new_records = {}

    def make_record(self, index):
        """Creates a record with the form 'eventlet-%s-%d'."""
        uuid = "eventlet-%s-%d" % (self.test_uuid, index)
        instance = {'uuid': uuid}
        entry = self.entry_factory.create_entry(instance)
        entry.name = uuid + "." + self.entry_factory.default_dns_zone.name
        entry.content = "123.123.123.123"
        self.driver.create_entry(entry)
        self.new_records[index] = True

    @test(enabled=WHITE_BOX and RUN_DNS)
    def use_dns_from_a_single_thread(self):
        """Add DNS records one at a time."""
        self.new_records = {}
        for index in range(-1, -5, -1):
            self.make_record(index)
            self.assert_record_created(index)

    @test(enabled=WHITE_BOX and RUN_DNS)
    def use_dns_from_multiple_greenthreads(self):
        """Add multiple DNS records at once."""
        self.new_records = {}

        def make_record(index):
            def __cb():
                self.make_record(index)
                self.assert_record_created(index)
                return index
            return __cb

        pile = eventlet.GreenPile()
        indices = range(1, 4)
        for index in indices:
            pile.spawn(make_record(index))

        list(pile)  # Wait for them to finish
        for index in indices:
            self.assert_record_created(index)



# Copyright 2013 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from oslo_log import log as logging

from trove.common.strategies.strategy import Strategy

LOG = logging.getLogger(__name__)


def get_storage_strategy(storage_driver, ns=__name__):
    LOG.debug("Getting storage strategy: %s.", storage_driver)
    return Strategy.get_strategy(storage_driver, ns)



# Copyright [2015] Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jsonschema

from mock import Mock, patch, MagicMock, PropertyMock
from testtools.matchers import Is, Equals

from trove.common import exception
from trove.common import glance_remote
from trove.datastore import models as datastore_models
from trove.extensions.mgmt.datastores.service import DatastoreVersionController
from trove.tests.unittests import trove_testtools


class TestDatastoreVersionController(trove_testtools.TestCase):
    def setUp(self):
        super(TestDatastoreVersionController, self).setUp()
        self.controller = DatastoreVersionController()

        self.version = {
            "version": {
                "datastore_name": "test_dsx",
                "name": "test_vr1",
                "datastore_manager": "mysql",
                "image": "154b350d-4d86-4214-9067-9c54b230c0da",
                "packages": ["mysql-server-5.7"],
                "active": True,
                "default": False
            }
        }

        self.tenant_id = Mock()
        context = trove_testtools.TroveTestContext(self)
        self.req = Mock()
        self.req.environ = Mock()
        self.req.environ.__getitem__ = Mock(return_value=context)

    def test_get_schema_create(self):
        schema = self.controller.get_schema('create', self.version)
        self.assertIsNotNone(schema)
        self.assertIn('version', schema['properties'])

    def test_validate_create(self):
        body = self.version
        schema = self.controller.get_schema('create', body)
        validator = jsonschema.Draft4Validator(schema)
        self.assertTrue(validator.is_valid(body))

    def test_validate_create_blankname(self):
        body = self.version
        body['version']['name'] = "     "
        schema = self.controller.get_schema('create', body)
        validator = jsonschema.Draft4Validator(schema)
        self.assertFalse(validator.is_valid(body))
        errors = sorted(validator.iter_errors(body), key=lambda e: e.path)
        self.assertThat(len(errors), Is(1))
        self.assertThat(errors[0].message,
                        Equals("'     ' does not match '^.*[0-9a-zA-Z]+.*$'"))

    def test_validate_create_blank_datastore(self):
        body = self.version
        body['version']['datastore_name'] = ""
        schema = self.controller.get_schema('create', body)
        validator = jsonschema.Draft4Validator(schema)
        self.assertFalse(validator.is_valid(body))
        errors = sorted(validator.iter_errors(body), key=lambda e: e.path)
        error_messages = [error.message for error in errors]
        self.assertThat(len(errors), Is(2))
        self.assertIn("'' is too short", error_messages)
        self.assertIn("'' does not match '^.*[0-9a-zA-Z]+.*$'", error_messages)

    @patch.object(glance_remote, 'create_glance_client')
    @patch.object(datastore_models.Datastore, 'load')
    @patch.object(datastore_models.DatastoreVersion, 'load',
                  side_effect=exception.DatastoreVersionNotFound)
    @patch.object(datastore_models, 'update_datastore_version')
    def test_create_datastore_versions(self, mock_ds_version_create,
                                       mock_ds_version_load,
                                       mock_ds_load, mock_glance_client):
        body = self.version
        mock_ds_load.return_value.name = 'test_dsx'

        self.controller.create(self.req, body, self.tenant_id)
        mock_ds_version_create.assert_called_with(
            'test_dsx', 'test_vr1', 'mysql',
            '154b350d-4d86-4214-9067-9c54b230c0da',
            'mysql-server-5.7', True)

    @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
    def test_show_ds_version(self, mock_ds_version_load):
        id = Mock()

        self.controller.show(self.req, self.tenant_id, id)
        mock_ds_version_load.assert_called_with(id)

    @patch.object(datastore_models.Datastore, 'load')
    @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
    def test_delete_ds_version(self, mock_ds_version_load, mock_ds_load):
        ds_version_id = Mock()
        ds_version = Mock()
        mock_ds_version_load.return_value = ds_version
        self.controller.delete(self.req, self.tenant_id, ds_version_id)
        ds_version.delete.assert_called_with()

    @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
    @patch.object(datastore_models.DatastoreVersions, 'load_all')
    def test_index_ds_version(self, mock_ds_version_load_all,
                              mock_ds_version_load_by_uuid):
        mock_id = Mock()
        mock_ds_version = Mock()
        mock_ds_version.id = mock_id
        mock_ds_version_load_all.return_value = [mock_ds_version]

        self.controller.index(self.req, self.tenant_id)
        mock_ds_version_load_all.assert_called_with(only_active=False)
        mock_ds_version_load_by_uuid.assert_called_with(mock_id)

    @patch.object(glance_remote, 'create_glance_client')
    @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
    @patch.object(datastore_models, 'update_datastore_version')
    def test_edit_datastore_versions(self, mock_ds_version_update,
                                     mock_ds_version_load,
                                     mock_glance_client):
        body = {'image': '21c8805a-a800-4bca-a192-3a5a2519044d'}

        mock_ds_version = MagicMock()
        type(mock_ds_version).datastore_name = PropertyMock(
            return_value=self.version['version']['datastore_name'])
        type(mock_ds_version).name = PropertyMock(
            return_value=self.version['version']['name'])
        type(mock_ds_version).image_id = PropertyMock(
            return_value=self.version['version']['image'])
        type(mock_ds_version).packages = PropertyMock(
            return_value=self.version['version']['packages'])
        type(mock_ds_version).active = PropertyMock(
            return_value=self.version['version']['active'])
        type(mock_ds_version).manager = PropertyMock(
            return_value=self.version['version']['datastore_manager'])
        mock_ds_version_load.return_value = mock_ds_version

        self.controller.edit(self.req, body, self.tenant_id, Mock())
        mock_ds_version_update.assert_called_with(
            'test_dsx', 'test_vr1', 'mysql',
            '21c8805a-a800-4bca-a192-3a5a2519044d',
            'mysql-server-5.7', True)



# Copyright 2012 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


GROUP = "dbaas.api.instances.status"

from proboscis import before_class
from proboscis import test
from proboscis.asserts import assert_equal

from trove.tests.config import CONFIG
from trove.tests.util import create_dbaas_client
from trove.tests.util.users import Requirements
from trove.common.utils import poll_until


@test(groups=[GROUP])
class InstanceStatusTests(object):

    @before_class
    def set_up(self):
        reqs = Requirements(is_admin=False)
        self.user = CONFIG.users.find_user(reqs)
        self.dbaas = create_dbaas_client(self.user)

    @test
    def test_create_failure_on_volume_prov_failure(self):
        # Fake nova will fail a volume of size 9.
        response = self.dbaas.instances.create('volume_fail', 1,
            {'size': 9}, [])
        poll_until(lambda: self.dbaas.instances.get(response.id),
                   lambda instance: instance.status == 'ERROR',
                   time_out=10)
        instance = self.dbaas.instances.get(response.id)
        print("Status: %s" % instance.status)
        assert_equal(instance.status, "ERROR",
            "Instance did not drop to error after volume prov failure.")

    @test
    def test_create_failure_on_server_failure(self):
        # Fake nova will fail a server ending with 'SERVER_ERROR'."
        response = self.dbaas.instances.create('test_SERVER_ERROR', 1,
            {'size': 1}, [])
        poll_until(lambda: self.dbaas.instances.get(response.id),
                   lambda instance: instance.status == 'ERROR',
                   time_out=10)
        instance = self.dbaas.instances.get(response.id)
        print("Status: %s" % instance.status)
        assert_equal(instance.status, "ERROR",
            "Instance did not drop to error after server prov failure.")

    ###TODO(ed-): We don't at present have a way to test DNS in FAKE_MODE.
    @test(enabled=False)
    def test_create_failure_on_dns_failure(self):
        #TODO(ed-): Throw DNS-specific monkeywrench into works
        response = self.dbaas.instances.create('test_DNS_ERROR', 1,
            {'size': 1}, [])
        poll_until(lambda: self.dbaas.instances.get(response.id),
                   lambda instance: instance.status == 'ERROR',
                   time_out=10)
        instance = self.dbaas.instances.get(response.id)
        print("Status: %s" % instance.status)
        assert_equal(instance.status, "ERROR",
            "Instance did not drop to error after DNS prov failure.")




from proboscis.asserts import assert_equal
from proboscis import test
from proboscis import before_class

from trove.common.utils import poll_until
from trove.tests.util import create_client


class InstanceGenerator(object):

    def __init__(self, client, status=None, name=None, flavor=None,
                 account_id=None, created_at=None, databases=None, users=None,
                 volume_size=None):
        self.client = client
        self.status = status
        self.name = name
        self.flavor = flavor
        self.account_id = account_id
        self.databases = databases
        self.users = users
        self.volume_size = volume_size
        self.id = None

    def create_instance(self):
        #make the call to create the instance
        instance = self.client.instances.create(self.name, self.flavor,
                                self.volume_size, self.databases, self.users)
        self.client.assert_http_code(200)

        #verify we are in a build state
        assert_equal(instance.status, "BUILD")
        #pull out the ID
        self.id = instance.id

        return instance

    def wait_for_build_to_finish(self):
        poll_until(lambda: self.client.instance.get(self.id),
                   lambda instance: instance.status != "BUILD",
                   time_out=600)

    def get_active_instance(self):
        instance = self.client.instance.get(self.id)
        self.client.assert_http_code(200)

        #check the container name
        assert_equal(instance.name, self.name)

        #pull out volume info and verify
        assert_equal(str(instance.volume_size), str(self.volume_size))

        #pull out the flavor and verify
        assert_equal(str(instance.flavor), str(self.flavor))

        return instance


@test(groups=['smoke', 'positive'])
class CreateInstance(object):

    @before_class
    def set_up(self):
        client = create_client(is_admin=False)
        name = 'test_createInstance_container'
        flavor = 1
        volume_size = 1
        db_name = 'test_db'
        databases = [
            {
                "name": db_name
            }
        ]
        users = [
            {
                "name": "lite",
                "password": "litepass",
                "databases": [{"name": db_name}]
            }
        ]

        #create the Instance
        instance = InstanceGenerator(client, name=self.name,
                                     flavor=flavor,
                                     volume_size=self.volume_size,
                                     databases=databases, users=users)
        instance.create_instance()

        #wait for the instance
        instance.wait_for_build_to_finish()

        #get the active instance
        inst = instance.get_active_instance()

        #list out the databases for our instance and verify the db name
        dbs = client.databases.list(inst.id)
        client.assert_http_code(200)

        assert_equal(len(dbs), 1)
        assert_equal(dbs[0].name, instance.db_name)

        client.instance.delete(inst.id)
        client.assert_http_code(202)



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Checks that the domain specified in the flag file exists and is valid.

If you define the environment variable ADD_DOMAINS=True when running the tests,
they will create the domain if its not found (see below for details).

"""
import time
from proboscis import test
from proboscis import before_class
from proboscis.asserts import assert_equal
from proboscis.asserts import assert_not_equal
from proboscis.decorators import time_out

from trove.tests.config import CONFIG

WHITE_BOX = CONFIG.white_box
RUN_DNS = CONFIG.values.get("trove_dns_support", False)

if WHITE_BOX:
    from nova import utils
    from nova import flags
    import rsdns
    from trove.dns.rsdns.driver import create_client_with_flag_values
    from trove.dns.driver import DnsEntry
    from trove.dns.rsdns.driver import RsDnsInstanceEntryFactory
    from trove.dns.rsdns.driver import RsDnsDriver
    from trove.dns.rsdns.driver import RsDnsZone
    from trove.utils import poll_until
    FLAGS = flags.FLAGS
    TEST_CONTENT = "126.1.1.1"
    TEST_NAME = "hiwassup.%s" % FLAGS.dns_domain_name
    DNS_DOMAIN_ID = None


@test(groups=["rsdns.domains", "rsdns.show_entries"],
      enabled=WHITE_BOX and RUN_DNS)
class ClientTests(object):

    @before_class
    def increase_logging(self):
        import httplib2
        httplib2.debuglevel = 1

    @test
    def can_auth(self):
        self.client = create_client_with_flag_values()
        self.client.authenticate()

    @test(depends_on=[can_auth])
    def list_domains(self):
        domains = self.client.domains.list()
        print(domains)


@test(groups=["rsdns.domains"], depends_on=[ClientTests],
      enabled=WHITE_BOX and RUN_DNS)
class RsDnsDriverTests(object):
    """Tests the RS DNS Driver."""

    def create_domain_if_needed(self):
        """Adds the domain specified in the flags."""
        print("Creating domain %s" % self.driver.default_dns_zone.name)
        future = self.driver.dns_client.domains.create(
            self.driver.default_dns_zone.name)
        while not future.ready:
            time.sleep(2)
        print("Got something: %s" % future.resource)
        with open('/home/vagrant/dns_resource.txt', 'w') as f:
            f.write('%r\n' % future.result[0].id)
        global DNS_DOMAIN_ID
        DNS_DOMAIN_ID = future.result[0].id
        print("The domain should have been created with id=%s" % DNS_DOMAIN_ID)

    @test
    @time_out(2 * 60)
    def ensure_domain_specified_in_flags_exists(self):
        """Make sure the domain in the FLAGS exists."""
        self.driver = RsDnsDriver(raise_if_zone_missing=False)
        assert_not_equal(None, self.driver.default_dns_zone)

        def zone_found():
            zones = self.driver.get_dns_zones()
            print("Retrieving zones.")
            for zone in zones:
                print("zone %s" % zone)
                if zone.name == self.driver.default_dns_zone.name:
                    self.driver.default_dns_zone.id = zone.id
                    global DNS_DOMAIN_ID
                    DNS_DOMAIN_ID = zone.id
                    return True
            return False
        if zone_found():
            return
        self.create_domain_if_needed()
        for i in range(5):
            if zone_found():
                return
        self.fail("""Could not find default dns zone.
                  This happens when they clear the staging DNS service of data.
                  To fix it, manually run the tests as follows:
                  $ ADD_DOMAINS=True python int_tests.py
                  and if all goes well the tests will create a new domain
                  record.""")

    @test(depends_on=[ensure_domain_specified_in_flags_exists],
          enabled=WHITE_BOX and FLAGS.dns_domain_name != "dbaas.rackspace.com")
    def delete_all_entries(self):
        """Deletes all entries under the default domain."""
        list = self.driver.get_entries()
        for entry in list:
            if entry.type == "A":
                self.driver.delete_entry(name=entry.name, type=entry.type,
                                         dns_zone=entry.dns_zone)
        # It takes awhile for them to be deleted.
        poll_until(lambda: self.driver.get_entries_by_name(TEST_NAME),
                   lambda list: len(list) == 0,
                   sleep_time=4, time_out=60)

    @test(depends_on=[delete_all_entries])
    def create_test_entry(self):
        fullname = TEST_NAME
        entry = DnsEntry(name=fullname, content=TEST_CONTENT, type="A",
                         ttl=3600)
        self.driver.create_entry(entry)
        list = None
        for i in range(500):
            list = self.driver.get_entries_by_name(name=fullname)
            if len(list) > 0:
                break
            time.sleep(1)
        print("This is the list: %r" % list)
        assert_equal(1, len(list))
        list2 = self.driver.get_entries_by_content(content=TEST_CONTENT)
        assert_equal(1, len(list2))

    @test(depends_on=[delete_all_entries])
    def create_test_rsdns_entry(self):
        """Create an entry using the RsDnsInstanceEntryFactory."""
        instance = {'uuid': '000136c0-effa-4711-a747-a5b9fbfcb3bd', 'id': '10'}
        ip = "10.100.2.7"
        factory = RsDnsInstanceEntryFactory(dns_domain_id=DNS_DOMAIN_ID)
        entry = factory.create_entry(instance)
        entry.content = ip
        self.driver.create_entry(entry)
        entries = self.driver.get_entries_by_name(name=entry.name)
        assert_equal(1, len(entries))
        assert_equal(ip, entries[0].content)
        assert_equal(FLAGS.dns_ttl, entries[0].ttl)

    @test(depends_on=[create_test_entry])
    def delete_test_entry(self):
        fullname = TEST_NAME
        self.driver.delete_entry(fullname, "A")
        # It takes awhile for them to be deleted.
        poll_until(lambda: self.driver.get_entries_by_name(TEST_NAME),
                   lambda list: len(list) == 0,
                   sleep_time=2, time_out=60)



# Copyright 2015 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from mock import DEFAULT
from mock import MagicMock
from mock import patch
from testtools.matchers import Is, Equals, Not

from trove.common.instance import ServiceStatuses
from trove.guestagent import backup
from trove.guestagent.common import configuration
from trove.guestagent.common.configuration import ImportOverrideStrategy
from trove.guestagent.common import operating_system
from trove.guestagent.datastore.experimental.db2 import (
    manager as db2_manager)
from trove.guestagent.datastore.experimental.db2 import (
    service as db2_service)
from trove.guestagent import pkg
from trove.guestagent import volume
from trove.tests.unittests.guestagent.test_datastore_manager import \
    DatastoreManagerTest


class GuestAgentDB2ManagerTest(DatastoreManagerTest):

    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
                    chown=DEFAULT, chmod=DEFAULT)
    @patch.object(db2_service.DB2App, 'process_default_dbm_config')
    def setUp(self, *arg, **kwargs):
        super(GuestAgentDB2ManagerTest, self).setUp('db2')
        self.real_status = db2_service.DB2AppStatus.set_status

        class FakeInstanceServiceStatus(object):
            status = ServiceStatuses.NEW

            def save(self):
                pass

        db2_service.DB2AppStatus.set_status = MagicMock(
            return_value=FakeInstanceServiceStatus())
        self.manager = db2_manager.Manager()
        self.real_db_app_status = db2_service.DB2AppStatus
        self.origin_format = volume.VolumeDevice.format
        self.origin_mount = volume.VolumeDevice.mount
        self.origin_mount_points = volume.VolumeDevice.mount_points
        self.origin_stop_db = db2_service.DB2App.stop_db
        self.origin_start_db = db2_service.DB2App.start_db
        self.orig_change_ownership = (db2_service.DB2App.change_ownership)
        self.orig_create_databases = db2_service.DB2Admin.create_database
        self.orig_list_databases = db2_service.DB2Admin.list_databases
        self.orig_delete_database = db2_service.DB2Admin.delete_database
        self.orig_create_users = db2_service.DB2Admin.create_user
        self.orig_list_users = db2_service.DB2Admin.list_users
        self.orig_delete_user = db2_service.DB2Admin.delete_user
        self.orig_update_hostname = db2_service.DB2App.update_hostname
        self.orig_backup_restore = backup.restore
        self.orig_init_config = db2_service.DB2App.init_config
        self.orig_update_overrides = db2_service.DB2App.update_overrides
        self.orig_remove_overrides = db2_service.DB2App.remove_overrides

    def tearDown(self):
        super(GuestAgentDB2ManagerTest, self).tearDown()
        db2_service.DB2AppStatus.set_status = self.real_db_app_status
        volume.VolumeDevice.format = self.origin_format
        volume.VolumeDevice.mount = self.origin_mount
        volume.VolumeDevice.mount_points = self.origin_mount_points
        db2_service.DB2App.stop_db = self.origin_stop_db
        db2_service.DB2App.start_db = self.origin_start_db
        db2_service.DB2App.change_ownership = self.orig_change_ownership
        db2_service.DB2Admin.create_database = self.orig_create_databases
        db2_service.DB2Admin.create_user = self.orig_create_users
        db2_service.DB2Admin.create_database = self.orig_create_databases
        db2_service.DB2Admin.list_databases = self.orig_list_databases
        db2_service.DB2Admin.delete_database = self.orig_delete_database
        db2_service.DB2Admin.create_user = self.orig_create_users
        db2_service.DB2Admin.list_users = self.orig_list_users
        db2_service.DB2Admin.delete_user = self.orig_delete_user
        db2_service.DB2App.update_hostname = self.orig_update_hostname
        backup.restore = self.orig_backup_restore
        db2_service.DB2App.init_config = self.orig_init_config
        db2_service.DB2App.update_overrides = self.orig_update_overrides
        db2_service.DB2App.remove_overrides = self.orig_remove_overrides

    def test_update_status(self):
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        self.manager.update_status(self.context)
        mock_status.update.assert_any_call()

    def test_prepare_device_path_true(self):
        self._prepare_dynamic()

    def test_prepare_device_path_false(self):
        self._prepare_dynamic(device_path=None)

    def test_prepare_database(self):
        self._prepare_dynamic(databases=['db1'])

    def test_prepare_from_backup(self):
        self._prepare_dynamic(['db2'], backup_id='123backup')

    @patch.object(configuration.ConfigurationManager, 'save_configuration')
    def _prepare_dynamic(self, packages=None, databases=None, users=None,
                         config_content='MockContent', device_path='/dev/vdb',
                         is_db_installed=True, backup_id=None, overrides=None):

        backup_info = {'id': backup_id,
                       'location': 'fake-location',
                       'type': 'DB2Backup',
                       'checksum': 'fake-checksum'} if backup_id else None

        mock_status = MagicMock()
        mock_app = MagicMock()
        self.manager.appStatus = mock_status
        self.manager.app = mock_app

        mock_status.begin_install = MagicMock(return_value=None)
        mock_app.change_ownership = MagicMock(return_value=None)
        mock_app.restart = MagicMock(return_value=None)
        mock_app.start_db = MagicMock(return_value=None)
        mock_app.stop_db = MagicMock(return_value=None)
        volume.VolumeDevice.format = MagicMock(return_value=None)
        volume.VolumeDevice.mount = MagicMock(return_value=None)
        volume.VolumeDevice.mount_points = MagicMock(return_value=[])
        db2_service.DB2Admin.create_user = MagicMock(return_value=None)
        db2_service.DB2Admin.create_database = MagicMock(return_value=None)
        backup.restore = MagicMock(return_value=None)

        with patch.object(pkg.Package, 'pkg_is_installed',
                          return_value=MagicMock(
                              return_value=is_db_installed)):
            self.manager.prepare(context=self.context, packages=packages,
                                 config_contents=config_content,
                                 databases=databases,
                                 memory_mb='2048', users=users,
                                 device_path=device_path,
                                 mount_point="/home/db2inst1/db2inst1",
                                 backup_info=backup_info,
                                 overrides=None,
                                 cluster_config=None)

        mock_status.begin_install.assert_any_call()
        self.assertEqual(1, mock_app.change_ownership.call_count)
        if databases:
            self.assertTrue(db2_service.DB2Admin.create_database.called)
        else:
            self.assertFalse(db2_service.DB2Admin.create_database.called)

        if users:
            self.assertTrue(db2_service.DB2Admin.create_user.called)
        else:
            self.assertFalse(db2_service.DB2Admin.create_user.called)

        if backup_id:
            backup.restore.assert_any_call(self.context,
                                           backup_info,
                                           '/home/db2inst1/db2inst1')
        self.assertTrue(
            self.manager.configuration_manager.save_configuration.called
        )

    def test_restart(self):
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        with patch.object(db2_service.DB2App, 'restart',
                          return_value=None) as restart_mock:
            # invocation
            self.manager.restart(self.context)
            # verification/assertion
            restart_mock.assert_any_call()

    def test_stop_db(self):
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        db2_service.DB2App.stop_db = MagicMock(return_value=None)
        self.manager.stop_db(self.context)
        db2_service.DB2App.stop_db.assert_any_call(
            do_not_start_on_reboot=False)

    def test_start_db_with_conf_changes(self):
        with patch.object(db2_service.DB2App, 'start_db_with_conf_changes'):
            self.manager.start_db_with_conf_changes(self.context, 'something')
            db2_service.DB2App.start_db_with_conf_changes.assert_any_call(
                'something')

    def test_create_database(self):
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        db2_service.DB2Admin.create_database = MagicMock(return_value=None)
        self.manager.create_database(self.context, ['db1'])
        db2_service.DB2Admin.create_database.assert_any_call(['db1'])

    def test_create_user(self):
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        db2_service.DB2Admin.create_user = MagicMock(return_value=None)
        self.manager.create_user(self.context, ['user1'])
        db2_service.DB2Admin.create_user.assert_any_call(['user1'])

    def test_delete_database(self):
        databases = ['db1']
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        db2_service.DB2Admin.delete_database = MagicMock(return_value=None)
        self.manager.delete_database(self.context, databases)
        db2_service.DB2Admin.delete_database.assert_any_call(databases)

    def test_delete_user(self):
        user = ['user1']
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        db2_service.DB2Admin.delete_user = MagicMock(return_value=None)
        self.manager.delete_user(self.context, user)
        db2_service.DB2Admin.delete_user.assert_any_call(user)

    def test_list_databases(self):
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        db2_service.DB2Admin.list_databases = MagicMock(
            return_value=['database1'])
        databases = self.manager.list_databases(self.context)
        self.assertThat(databases, Not(Is(None)))
        self.assertThat(databases, Equals(['database1']))
        db2_service.DB2Admin.list_databases.assert_any_call(None, None, False)

    def test_list_users(self):
        db2_service.DB2Admin.list_users = MagicMock(return_value=['user1'])
        users = self.manager.list_users(self.context)
        self.assertThat(users, Equals(['user1']))
        db2_service.DB2Admin.list_users.assert_any_call(None, None, False)

    @patch.object(db2_service.DB2Admin, 'get_user',
                  return_value=MagicMock(return_value=['user1']))
    def test_get_users(self, get_user_mock):
        username = ['user1']
        hostname = ['host']
        mock_status = MagicMock()
        self.manager.appStatus = mock_status
        users = self.manager.get_user(self.context, username, hostname)
        self.assertThat(users, Equals(get_user_mock.return_value))
        get_user_mock.assert_any_call(username, hostname)

    def test_rpc_ping(self):
        output = self.manager.rpc_ping(self.context)
        self.assertTrue(output)

    def test_update_update_overrides(self):
        configuration = {"DIAGSIZE": 50}
        db2_service.DB2App.update_overrides = MagicMock()
        self.manager.update_overrides(self.context, configuration, False)
        db2_service.DB2App.update_overrides.assert_any_call(self.context,
                                                            configuration)

    def test_reset_update_overrides(self):
        configuration = {}
        db2_service.DB2App.remove_overrides = MagicMock()
        self.manager.update_overrides(self.context, configuration, True)
        db2_service.DB2App.remove_overrides.assert_any_call()



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from proboscis import test

from trove.tests.config import CONFIG
from trove.tests.util import create_dbaas_client
from trove.tests.util.users import Requirements

GROUP = "dbaas.api.instances.delete"


@test(groups=[GROUP])
def delete_all():
    """Delete every single one."""
    user = CONFIG.users.find_user(Requirements(is_admin=False))
    dbaas = create_dbaas_client(user)
    instances = dbaas.instances.list()
    for instance in instances:
        instance.delete()



# Copyright [2016] Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from trove.common import cfg
from trove.common import exception
from trove.extensions.common.service import ClusterRootController

CONF = cfg.CONF
MANAGER = CONF.datastore_manager if CONF.datastore_manager else 'pxc'


class PxcRootController(ClusterRootController):

    def root_delete(self, req, tenant_id, instance_id, is_cluster):
        raise exception.DatastoreOperationNotSupported(
            operation='disable_root', datastore=MANAGER)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="Trove Integration Tests",
    version="0.0.9.9",
    author='OpenStack',
    description="Runs integration tests on Ridley.",
    license='Apache',
    py_modules=[],
    packages=['tests'],
    scripts=[]
)



# Copyright 2011 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Tests classes which convert RS style-entries to Nova DNS entries."""

import hashlib
import unittest
from proboscis import test

from trove.tests.config import CONFIG


if CONFIG.white_box:
    from nova import flags
    from rsdns.client.records import Record
    from trove.dns.rsdns.driver import EntryToRecordConverter
    from trove.dns.rsdns.driver import RsDnsInstanceEntryFactory
    from trove.dns.rsdns.driver import RsDnsZone
    FLAGS = flags.FLAGS
    driver = None
    DEFAULT_ZONE = RsDnsZone(1, "dbaas.rackspace.org")
    TEST_CONTENT = "126.1.1.1"
    TEST_NAME = "hiwassup.dbaas.rackspace.org"


@test(groups=["unit", "rsdns.conversion"],
      enabled=CONFIG.white_box)
class ConvertingNovaEntryNamesToRecordNames(unittest.TestCase):

    def setUp(self):
        self.converter = EntryToRecordConverter(DEFAULT_ZONE)
        self.fake_zone = RsDnsZone(id=5, name="blah.org")

    def test_normal_name(self):
        long_name = self.converter.name_to_long_name("hi", self.fake_zone)
        self.assertEqual("hi.blah.org", long_name)

    def test_short_name(self):
        long_name = self.converter.name_to_long_name("", self.fake_zone)
        self.assertEqual("", long_name)

    def test_long_name(self):
        long_name = self.converter.name_to_long_name("blah.org.",
                                                     self.fake_zone)
        self.assertEqual("blah.org..blah.org", long_name)


@test(groups=["unit", "rsdns.conversion"],
      enabled=CONFIG.white_box)
class ConvertingRecordsToEntries(unittest.TestCase):

    def setUp(self):
        self.converter = EntryToRecordConverter(DEFAULT_ZONE)
        self.fake_zone = RsDnsZone(id=5, name="blah.org")

    def test_normal_name(self):
        record = Record(None, {"id": 5, "name": "hi.blah.org",
                               "data": "stacker.com blah@blah 13452378",
                               "ttl": 5,
                               "type": "SOA"})
        entry = self.converter.record_to_entry(record=record,
                                               dns_zone=self.fake_zone)
        self.assertEqual("stacker.com blah@blah 13452378", entry.content)
        self.assertEqual("hi.blah.org", entry.name)
        self.assertEqual("5", str(entry.ttl))
        self.assertEqual("SOA", entry.type)


@test(groups=["rsdns.conversion"],
      enabled=CONFIG.white_box)
class WhenCreatingAnEntryForAnInstance(unittest.TestCase):
    # This isn't a unit test because RsDnsInstanceEntryFactory connects to the
    # service.

    def setUp(self):
        self.creator = RsDnsInstanceEntryFactory()

    def test_should_concatanate_strings(self):
        instance = {'id': '56',
                    'uuid': '000136c0-effa-4711-a747-a5b9fbfcb3bd'}
        entry = self.creator.create_entry(instance)
        expected_name = "%s.%s" % (hashlib.sha1(instance['uuid']).hexdigest(),
                                   FLAGS.dns_domain_name)
        self.assertEqual(expected_name, entry.name,
                         msg="Entry name should match - %s" % entry.name)
        self.assertIsNone(entry.content)
        self.assertEqual("A", entry.type)
        self.assertEqual(FLAGS.dns_ttl, entry.ttl)
        self.assertIsNone(entry.priority)
        self.assertEqual(FLAGS.dns_domain_name, entry.dns_zone.name)
        if not entry.dns_zone.id:
            self.fail(msg="DNS Zone Id should not be empty")




from proboscis import after_class
from proboscis import before_class
from proboscis import test
from proboscis.asserts import assert_equal
from proboscis.asserts import assert_is_not
from proboscis.asserts import assert_is_none
from proboscis.asserts import assert_true


from troveclient.compat import exceptions
from trove.tests.config import CONFIG
from trove.tests.util import create_dbaas_client
from trove.tests.util.users import Requirements


class TestBase(object):

    def set_up(self):
        """Create a ton of instances."""
        reqs = Requirements(is_admin=False)
        self.user = CONFIG.users.find_user(reqs)
        self.dbaas = create_dbaas_client(self.user)

    def delete_instances(self):
        chunk = 0
        while True:
            chunk += 1
            attempts = 0
            instances = self.dbaas.instances.list()
            if len(instances) == 0:
                break
            # Sit around and try to delete this chunk.
            while True:
                instance_results = []
                attempts += 1
                deleted_count = 0
                for instance in instances:
                    try:
                        instance.delete()
                        result = "[w]"
                    except exceptions.UnprocessableEntity:
                        result = "[W]"
                    except exceptions.NotFound:
                        result = "[O]"
                        deleted_count += 1
                    except Exception:
                        result = "[X]"
                    instance_results.append(result)
                print("Chunk %d, attempt %d : %s"
                      % (chunk, attempts, ",".join(instance_results)))
                if deleted_count == len(instances):
                    break

    def create_instances(self):
        self.ids = []
        for index in range(self.max):
            name = "multi-%03d" % index
            result = self.dbaas.instances.create(name, 1,
                                                 {'size': 1}, [], [])
            self.ids.append(result.id)
        # Sort the list of IDs in order, so we can confirm the lists pagination
        # returns is also sorted correctly.
        self.ids.sort()

    @staticmethod
    def assert_instances_sorted_by_ids(instances):
        # Assert that the strings are always increasing.
        last_id = ""
        for instance in instances:
            assert_true(last_id < instance.id)

    def print_list(self, instances):
        print("Length = %d" % len(instances))
        print(",".join([instance.id for instance in instances]))

    def test_pagination(self, requested_limit, requested_marker,
                        expected_length, expected_marker, expected_last_item):
        instances = self.dbaas.instances.list(limit=requested_limit,
                                              marker=requested_marker)
        marker = instances.next

        self.print_list(instances)

        # Better get as many as we asked for.
        assert_equal(len(instances), expected_length)
        # The last one should be roughly this one in the list.
        assert_equal(instances[-1].id, expected_last_item)
        # Because limit < count, the marker must be something.
        if expected_marker:
            assert_is_not(marker, None)
            assert_equal(marker, expected_marker)
        else:
            assert_is_none(marker)
        self.assert_instances_sorted_by_ids(instances)


@test(runs_after_groups=["dbaas.guest.shutdown"],
      groups=['dbaas.api.instances.pagination'])
class SimpleCreateAndDestroy(TestBase):
    """
    It turns out a big part of guaranteeing pagination works is to make sure
    we can create a big batch of instances and delete them without problems.
    Even in fake mode though its worth it to check this is the case.
    """

    max = 5

    @before_class
    def set_up(self):
        """Create a ton of instances."""
        super(SimpleCreateAndDestroy, self).set_up()
        self.delete_instances()

    @test
    def spin_up(self):
        self.create_instances()

    @after_class(always_run=True)
    def tear_down(self):
        self.delete_instances()


@test(runs_after_groups=["dbaas.guest.shutdown"],
      groups=['dbaas.api.instances.pagination'])
class InstancePagination50(TestBase):

    max = 50

    @before_class
    def set_up(self):
        """Create a ton of instances."""
        super(InstancePagination50, self).set_up()
        self.delete_instances()
        self.create_instances()

    @after_class(always_run=True)
    def tear_down(self):
        """Tear down all instances."""
        self.delete_instances()

    @test
    def pagination_short(self):
        self.test_pagination(requested_limit=10, requested_marker=None,
                             expected_length=10, expected_marker=self.ids[9],
                             expected_last_item=self.ids[9])

    @test
    def pagination_default(self):
        self.test_pagination(requested_limit=None, requested_marker=None,
                             expected_length=20, expected_marker=self.ids[19],
                             expected_last_item=self.ids[19])

    @test
    def pagination_full(self):
        self.test_pagination(requested_limit=50, requested_marker=None,
                             expected_length=20, expected_marker=self.ids[19],
                             expected_last_item=self.ids[19])


@test(runs_after_groups=["dbaas.guest.shutdown"],
      groups=['dbaas.api.instances.pagination'])
class InstancePagination20(TestBase):

    max = 20

    @before_class
    def set_up(self):
        """Create a ton of instances."""
        super(InstancePagination20, self).set_up()
        self.delete_instances()
        self.create_instances()

    @after_class(always_run=True)
    def tear_down(self):
        """Tear down all instances."""
        self.delete_instances()

    @test
    def pagination_short(self):
        self.test_pagination(requested_limit=10, requested_marker=None,
                             expected_length=10, expected_marker=self.ids[9],
                             expected_last_item=self.ids[9])

    @test
    def pagination_default(self):
        self.test_pagination(requested_limit=None, requested_marker=None,
                             expected_length=20, expected_marker=None,
                             expected_last_item=self.ids[19])

    @test
    def pagination_full(self):
        self.test_pagination(requested_limit=20, requested_marker=None,
                             expected_length=20, expected_marker=None,
                             expected_last_item=self.ids[19])

    @test
    def pagination_overkill(self):
        self.test_pagination(requested_limit=30, requested_marker=None,
                             expected_length=20, expected_marker=None,
                             expected_last_item=self.ids[19])

    @test
    def pagination_last_half(self):
        self.test_pagination(requested_limit=10, requested_marker=self.ids[9],
                             expected_length=10, expected_marker=None,
                             expected_last_item=self.ids[19])

    @test
    def pagination_third_quarter(self):
        self.test_pagination(requested_limit=5, requested_marker=self.ids[9],
                             expected_length=5, expected_marker=self.ids[14],
                             expected_last_item=self.ids[14])

    @test
    def pagination_fourth_quarter(self):
        self.test_pagination(requested_limit=20, requested_marker=self.ids[14],
                             expected_length=5, expected_marker=None,
                             expected_last_item=self.ids[19])



#    Copyright 2011 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from nose.plugins.skip import SkipTest
import proboscis
from proboscis.asserts import assert_equal
from proboscis.asserts import assert_false
from proboscis.asserts import assert_not_equal
from proboscis.asserts import assert_raises
from proboscis.asserts import assert_true
from proboscis import test
from troveclient.compat import exceptions

from trove.common import utils
from trove import tests
from trove.tests.api import instances
from trove.tests.config import CONFIG
from trove.tests import util
from trove.tests.util import test_config
from trove.tests.util import users as users_util

GROUP = "dbaas.api.root"


@test(groups=[tests.DBAAS_API, GROUP, tests.INSTANCES])
class TestRoot(object):
    """
    Test the root operations
    """

    root_enabled_timestamp = 'Never'
    system_users = ['root', 'debian_sys_maint']

    @proboscis.before_class
    def setUp(self):
        self.info = instances.InstanceTestInfo()

        reqs = users_util.Requirements(is_admin=True)
        self.info.admin_user = CONFIG.users.find_user(reqs)
        self.info.dbaas_admin = self.dbaas_admin = util.create_dbaas_client(
            self.info.admin_user
        )
        reqs = users_util.Requirements(is_admin=False)
        self.info.user = CONFIG.users.find_user(reqs)
        self.info.dbaas = self.dbaas = util.create_dbaas_client(self.info.user)

        self.info.name = "TEST_%s" % self.__class__.__name__

        flavor, flavor_href = self.info.find_default_flavor()
        self.info.dbaas_flavor = flavor
        self.info.dbaas_flavor_href = flavor_href

        databases = []
        databases.append({"name": "firstdb", "character_set": "latin2",
                          "collate": "latin2_general_ci"})
        databases.append({"name": "db2"})
        self.info.databases = databases

        users = []
        users.append({"name": "lite", "password": "litepass",
                      "databases": [{"name": "firstdb"}]})
        self.info.users = users

        self.info.dbaas_datastore = CONFIG.dbaas_datastore
        self.info.dbaas_datastore_version = CONFIG.dbaas_datastore_version
        self.info.volume = {'size': CONFIG.get('trove_volume_size', 2)}

        self.info.initial_result = self.dbaas.instances.create(
            self.info.name,
            self.info.dbaas_flavor_href,
            self.info.volume,
            databases,
            users,
            nics=self.info.nics,
            availability_zone="nova",
            datastore=self.info.dbaas_datastore,
            datastore_version=self.info.dbaas_datastore_version
        )

        assert_equal(200, self.dbaas.last_http_code)

        self.id = self.info.initial_result.id

        def result_is_active():
            instance = self.dbaas.instances.get(self.id)
            if instance.status == "ACTIVE":
                return True
            else:
                # If its not ACTIVE, anything but BUILD must be
                # an error.
                assert_equal("BUILD", instance.status)
                return False

        utils.poll_until(result_is_active)

    @proboscis.after_class
    def tearDown(self):
        self.dbaas.instances.delete(self.id)

        def _is_delete():
            try:
                self.dbaas.instances.get(self.id)
            except exceptions.NotFound:
                return True

        utils.poll_until(_is_delete)

    def _verify_root_timestamp(self, id):
        reh = self.dbaas_admin.management.root_enabled_history(id)
        timestamp = reh.enabled
        assert_equal(self.root_enabled_timestamp, timestamp)
        assert_equal(id, reh.id)

    def _root(self):
        self.dbaas.root.create(self.id)
        assert_equal(200, self.dbaas.last_http_code)
        reh = self.dbaas_admin.management.root_enabled_history
        self.root_enabled_timestamp = reh(self.id).enabled

    @test
    def test_root_initially_disabled(self):
        """Test that root is disabled."""
        enabled = self.dbaas.root.is_root_enabled(self.id)
        assert_equal(200, self.dbaas.last_http_code)

        is_enabled = enabled
        if hasattr(enabled, 'rootEnabled'):
            is_enabled = enabled.rootEnabled
        assert_false(is_enabled, "Root SHOULD NOT be enabled.")

    @test
    def test_create_user_os_admin_failure(self):
        users = [{"name": "os_admin", "password": "12345"}]
        assert_raises(exceptions.BadRequest, self.dbaas.users.create,
                      self.id, users)

    @test
    def test_delete_user_os_admin_failure(self):
        assert_raises(exceptions.BadRequest, self.dbaas.users.delete,
                      self.id, "os_admin")

    @test(depends_on=[test_root_initially_disabled],
          enabled=not test_config.values['root_removed_from_instance_api'])
    def test_root_initially_disabled_details(self):
        """Use instance details to test that root is disabled."""
        instance = self.dbaas.instances.get(self.id)
        assert_true(hasattr(instance, 'rootEnabled'),
                    "Instance has no rootEnabled property.")
        assert_false(instance.rootEnabled, "Root SHOULD NOT be enabled.")
        assert_equal(self.root_enabled_timestamp, 'Never')

    @test(depends_on=[test_root_initially_disabled_details])
    def test_root_disabled_in_mgmt_api(self):
        """Verifies in the management api that the timestamp exists."""
        self._verify_root_timestamp(self.id)

    @test(depends_on=[test_root_initially_disabled_details])
    def test_root_disable_when_root_not_enabled(self):
        reh = self.dbaas_admin.management.root_enabled_history
        self.root_enabled_timestamp = reh(self.id).enabled
        assert_raises(exceptions.NotFound, self.dbaas.root.delete,
                      self.id)
        self._verify_root_timestamp(self.id)

    @test(depends_on=[test_root_disable_when_root_not_enabled])
    def test_enable_root(self):
        self._root()

    @test(depends_on=[test_enable_root])
    def test_enabled_timestamp(self):
        assert_not_equal(self.root_enabled_timestamp, 'Never')

    @test(depends_on=[test_enable_root])
    def test_root_not_in_users_list(self):
        """
        Tests that despite having enabled root, user root doesn't appear
        in the users list for the instance.
        """
        users = self.dbaas.users.list(self.id)
        usernames = [user.name for user in users]
        assert_true('root' not in usernames)

    @test(depends_on=[test_enable_root])
    def test_root_now_enabled(self):
        """Test that root is now enabled."""
        enabled = self.dbaas.root.is_root_enabled(self.id)
        assert_equal(200, self.dbaas.last_http_code)
        assert_true(enabled, "Root SHOULD be enabled.")

    @test(depends_on=[test_root_now_enabled],
          enabled=not test_config.values['root_removed_from_instance_api'])
    def test_root_now_enabled_details(self):
        """Use instance details to test that root is now enabled."""
        instance = self.dbaas.instances.get(self.id)
        assert_true(hasattr(instance, 'rootEnabled'),
                    "Instance has no rootEnabled property.")
        assert_true(instance.rootEnabled, "Root SHOULD be enabled.")
        assert_not_equal(self.root_enabled_timestamp, 'Never')
        self._verify_root_timestamp(self.id)

    @test(depends_on=[test_root_now_enabled_details])
    def test_reset_root(self):
        if test_config.values['root_timestamp_disabled']:
            raise SkipTest("Enabled timestamp not enabled yet")
        old_ts = self.root_enabled_timestamp
        self._root()
        assert_not_equal(self.root_enabled_timestamp, 'Never')
        assert_equal(self.root_enabled_timestamp, old_ts)

    @test(depends_on=[test_reset_root])
    def test_root_still_enabled(self):
        """Test that after root was reset it's still enabled."""
        enabled = self.dbaas.root.is_root_enabled(self.id)
        assert_equal(200, self.dbaas.last_http_code)
        assert_true(enabled, "Root SHOULD still be enabled.")

    @test(depends_on=[test_root_still_enabled],
          enabled=not test_config.values['root_removed_from_instance_api'])
    def test_root_still_enabled_details(self):
        """Use instance details to test that after root was reset,
            it's still enabled.
        """
        instance = self.dbaas.instances.get(self.id)
        assert_true(hasattr(instance, 'rootEnabled'),
                    "Instance has no rootEnabled property.")
        assert_true(instance.rootEnabled, "Root SHOULD still be enabled.")
        assert_not_equal(self.root_enabled_timestamp, 'Never')
        self._verify_root_timestamp(self.id)

    @test(depends_on=[test_enable_root])
    def test_root_cannot_be_deleted(self):
        """Even if root was enabled, the user root cannot be deleted."""
        assert_raises(exceptions.BadRequest, self.dbaas.users.delete,
                      self.id, "root")

    @test(depends_on=[test_root_still_enabled_details])
    def test_root_disable(self):
        reh = self.dbaas_admin.management.root_enabled_history
        self.root_enabled_timestamp = reh(self.id).enabled
        self.dbaas.root.delete(self.id)
        assert_equal(204, self.dbaas.last_http_code)
        self._verify_root_timestamp(self.id)



# Copyright 2016 Tesora Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneauth1.identity import v3
from keystoneauth1 import session as ka_session

from oslo_utils.importutils import import_class

from trove.common import cfg
from trove.common.remote import get_endpoint
from trove.common.remote import normalize_url

from glanceclient import Client

CONF = cfg.CONF


def glance_client(context, region_name=None):

    # We should allow glance to get the endpoint from the service
    # catalog, but to do so we would need to be able to specify
    # the endpoint_filter on the API calls, but glance
    # doesn't currently allow that.  As a result, we must
    # specify the endpoint explicitly.
    if CONF.glance_url:
        endpoint_url = '%(url)s%(tenant)s' % {
            'url': normalize_url(CONF.glance_url),
            'tenant': context.project_id}
    else:
        endpoint_url = get_endpoint(
            context.service_catalog, service_type=CONF.glance_service_type,
            endpoint_region=region_name or CONF.os_region_name,
            endpoint_type=CONF.glance_endpoint_type)

    auth = v3.Token(CONF.trove_auth_url, context.auth_token)
    session = ka_session.Session(auth=auth)

    return Client(CONF.glance_client_version, endpoint=endpoint_url,
                  session=session)


def create_glance_client(*arg, **kwargs):
    return import_class(CONF.remote_glance_client)(*arg, **kwargs)




import unittest

from proboscis import test

from trove.tests.api.instances import instance_info
from trove.tests.api.instances import GROUP_START as INSTANCE_START
from trove.tests.api.instances import GROUP_TEST
from trove.tests.api.instances import GROUP_STOP as INSTANCE_STOP
from trove.tests.config import CONFIG
from trove.common.utils import import_object
from trove.common.utils import poll_until

WHITE_BOX = CONFIG.white_box

if WHITE_BOX:
    # TODO(tim.simpson): Restore this once white box functionality can be
    #                    added back to this test module.
    pass
    # import rsdns
    # from nova import flags
    # from nova import utils

    # from trove import exception
    # from trove.utils import poll_until

    # FLAGS = flags.FLAGS

dns_driver = None

GROUP = "dbaas.guest.dns"


@test(groups=[GROUP, GROUP_TEST])
class Setup(unittest.TestCase):
    """Creates the DNS Driver and entry factory used in subsequent tests."""

    def test_create_rs_dns_driver(self):
        global dns_driver
        dns_driver = import_object(FLAGS.dns_driver)


def expected_dns_entry():
    """Returns expected DNS entry for this instance.

    :rtype: Instance of :class:`DnsEntry`.

    """
    return create_dns_entry(instance_info.local_id, instance_info.id)


@test(depends_on_classes=[Setup],
      depends_on_groups=[INSTANCE_START],
      groups=[GROUP, GROUP_TEST])
class WhenInstanceIsCreated(unittest.TestCase):
    """Make sure the DNS name was provisioned.

    This class actually calls the DNS driver to confirm the entry that should
    exist for the given instance does exist.

    """

    def test_dns_entry_should_exist(self):
        entry = expected_dns_entry()
        if entry:
            def get_entries():
                return dns_driver.get_entries_by_name(entry.name)
            try:
                poll_until(get_entries, lambda entries: len(entries) > 0,
                           sleep_time=2, time_out=60)
            except exception.PollTimeOut:
                self.fail("Did not find name " + entry.name + \
                          " in the entries, which were as follows:"
                          + str(dns_driver.get_entries()))


@test(depends_on_classes=[Setup, WhenInstanceIsCreated],
      depends_on_groups=[INSTANCE_STOP],
      groups=[GROUP])
class AfterInstanceIsDestroyed(unittest.TestCase):
    """Make sure the DNS name is removed along with an instance.

    Because the compute manager calls the DNS manager with RPC cast, it can
    take awhile.  So we wait for 30 seconds for it to disappear.

    """

    def test_dns_entry_exist_should_be_removed_shortly_thereafter(self):
        entry = expected_dns_entry()

        if not entry:
            return

        def get_entries():
            return dns_driver.get_entries_by_name(entry.name)

        try:
            poll_until(get_entries, lambda entries: len(entries) == 0,
                       sleep_time=2, time_out=60)
        except exception.PollTimeOut:
            # Manually delete the rogue item
            dns_driver.delete_entry(entry.name, entry.type, entry.dns_zone)
            self.fail("The DNS entry was never deleted when the instance "
                      "was destroyed.")



# Copyright (c) 2013 eBay Software Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from oslo_log import log as logging

from trove.common import instance as rd_instance
from trove.common.notification import EndNotification
from trove.guestagent import backup
from trove.guestagent.datastore.experimental.couchbase import service
from trove.guestagent.datastore.experimental.couchbase import system
from trove.guestagent.datastore import manager
from trove.guestagent import volume


LOG = logging.getLogger(__name__)


class Manager(manager.Manager):
    """
    This is Couchbase Manager class. It is dynamically loaded
    based off of the datastore of the trove instance
    """
    def __init__(self):
        self.appStatus = service.CouchbaseAppStatus()
        self.app = service.CouchbaseApp(self.appStatus)
        super(Manager, self).__init__('couchbase')

    @property
    def status(self):
        return self.appStatus

    def reset_configuration(self, context, configuration):
        self.app.reset_configuration(configuration)

    def do_prepare(self, context, packages, databases, memory_mb, users,
                   device_path, mount_point, backup_info,
                   config_contents, root_password, overrides,
                   cluster_config, snapshot):
        """This is called from prepare in the base class."""
        self.app.install_if_needed(packages)
        if device_path:
            device = volume.VolumeDevice(device_path)
            # unmount if device is already mounted
            device.unmount_device(device_path)
            device.format()
            device.mount(mount_point)
            LOG.debug('Mounted the volume (%s).', device_path)
        self.app.start_db_with_conf_changes(config_contents)
        LOG.debug('Securing couchbase now.')
        self.app.initial_setup()
        if backup_info:
            LOG.debug('Now going to perform restore.')
            self._perform_restore(backup_info,
                                  context,
                                  mount_point)

    def restart(self, context):
        """
        Restart this couchbase instance.
        This method is called when the guest agent
        gets a restart message from the taskmanager.
        """
        self.app.restart()

    def start_db_with_conf_changes(self, context, config_contents):
        self.app.start_db_with_conf_changes(config_contents)

    def stop_db(self, context, do_not_start_on_reboot=False):
        """
        Stop this couchbase instance.
        This method is called when the guest agent
        gets a stop message from the taskmanager.
        """
        self.app.stop_db(do_not_start_on_reboot=do_not_start_on_reboot)

    def enable_root(self, context):
        LOG.debug("Enabling root.")
        return self.app.enable_root()

    def enable_root_with_password(self, context, root_password=None):
        return self.app.enable_root(root_password)

    def is_root_enabled(self, context):
        LOG.debug("Checking if root is enabled.")
        return os.path.exists(system.pwd_file)

    def _perform_restore(self, backup_info, context, restore_location):
        """
        Restores all couchbase buckets and their documents from the
        backup.
        """
        LOG.info("Restoring database from backup %s", backup_info['id'])
        try:
            backup.restore(context, backup_info, restore_location)
        except Exception as e:
            LOG.error("Error performing restore from backup %s",
                      backup_info['id'])
            LOG.error(e)
            self.status.set_status(rd_instance.ServiceStatuses.FAILED)
            raise
        LOG.info("Restored database successfully")

    def create_backup(self, context, backup_info):
        """
        Backup all couchbase buckets and their documents.
        """
        with EndNotification(context):
            backup.backup(context, backup_info)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import proboscis
from trove.tests.api import backups
from trove.tests.api import configurations
from trove.tests.api import databases
from trove.tests.api import datastores
from trove.tests.api import instances
from trove.tests.api import instances_actions
from trove.tests.api.mgmt import admin_required
from trove.tests.api.mgmt import datastore_versions
from trove.tests.api.mgmt import instances as mgmt_instances
from trove.tests.api import replication
from trove.tests.api import root
from trove.tests.api import user_access
from trove.tests.api import users
from trove.tests.api import versions
from trove.tests.scenario import groups
from trove.tests.scenario.groups import backup_group
from trove.tests.scenario.groups import cluster_group
from trove.tests.scenario.groups import configuration_group
from trove.tests.scenario.groups import database_actions_group
from trove.tests.scenario.groups import guest_log_group
from trove.tests.scenario.groups import instance_actions_group
from trove.tests.scenario.groups import instance_create_group
from trove.tests.scenario.groups import instance_delete_group
from trove.tests.scenario.groups import instance_error_create_group
from trove.tests.scenario.groups import instance_force_delete_group
from trove.tests.scenario.groups import instance_upgrade_group
from trove.tests.scenario.groups import module_group
from trove.tests.scenario.groups import negative_cluster_actions_group
from trove.tests.scenario.groups import replication_group
from trove.tests.scenario.groups import root_actions_group
from trove.tests.scenario.groups import user_actions_group


GROUP_SERVICES_INITIALIZE = "services.initialize"
GROUP_SETUP = 'dbaas.setup'


def build_group(*groups):
    def merge(collection, *items):
        for item in items:
            if isinstance(item, list):
                merge(collection, *item)
            else:
                if item not in collection:
                    collection.append(item)

    out = []
    merge(out, *groups)
    return out


def register(group_names, *test_groups, **kwargs):
    if kwargs:
        register(group_names, kwargs.values())
        for suffix, grp_set in kwargs.items():
            # Recursively call without the kwargs
            register([name + '_' + suffix for name in group_names], *grp_set)
        return

    # Do the actual registration here
    proboscis.register(groups=build_group(group_names),
                       depends_on_groups=build_group(*test_groups))
    # Now register the same groups with '-' instead of '_'
    proboscis.register(groups=build_group(
                       [name.replace('_', '-') for name in group_names]),
                       depends_on_groups=build_group(*test_groups))

black_box_groups = [
    users.GROUP,
    user_access.GROUP,
    databases.GROUP,
    root.GROUP,
    GROUP_SERVICES_INITIALIZE,
    instances.GROUP_START,
    instances.GROUP_QUOTAS,
    backups.BACKUP_GROUP,
    replication.REPLICATION_GROUP,
    configurations.CONFIGURATION_GROUP,
    instances_actions.GROUP_RESIZE,
    instances_actions.GROUP_STOP_MYSQL,
    instances.GROUP_STOP,
    instances.GROUP_GUEST,
    versions.GROUP,
    datastores.GROUP,
    datastore_versions.GROUP,
    # TODO(SlickNik): The restart tests fail intermittently so pulling
    # them out of the blackbox group temporarily. Refer to Trove bug:
    # https://bugs.launchpad.net/trove/+bug/1204233
    # instances_actions.GROUP_RESTART,
]
proboscis.register(groups=["blackbox", "mysql"],
                   depends_on_groups=black_box_groups)

simple_black_box_groups = [
    GROUP_SERVICES_INITIALIZE,
    versions.GROUP,
    instances.GROUP_START_SIMPLE,
    admin_required.GROUP,
    datastore_versions.GROUP,
]
proboscis.register(groups=["simple_blackbox"],
                   depends_on_groups=simple_black_box_groups)

black_box_mgmt_groups = [
    instances_actions.GROUP_REBOOT,
    admin_required.GROUP,
    mgmt_instances.GROUP,
    datastore_versions.GROUP,
]
proboscis.register(groups=["blackbox_mgmt"],
                   depends_on_groups=black_box_mgmt_groups)

#
# Group designations for datastore agnostic int-tests
#
# Base groups for all other groups
base_groups = [
    GROUP_SERVICES_INITIALIZE,
    versions.GROUP,
    GROUP_SETUP
]

# Cluster-based groups
cluster_create_groups = list(base_groups)
cluster_create_groups.extend([groups.CLUSTER_DELETE_WAIT])

cluster_actions_groups = list(cluster_create_groups)
cluster_actions_groups.extend([groups.CLUSTER_ACTIONS_SHRINK_WAIT])

cluster_negative_actions_groups = list(negative_cluster_actions_group.GROUP)

cluster_root_groups = list(cluster_create_groups)
cluster_root_groups.extend([groups.CLUSTER_ACTIONS_ROOT_ENABLE])

cluster_root_actions_groups = list(cluster_actions_groups)
cluster_root_actions_groups.extend([groups.CLUSTER_ACTIONS_ROOT_ACTIONS])

cluster_restart_groups = list(cluster_create_groups)
cluster_restart_groups.extend([groups.CLUSTER_ACTIONS_RESTART_WAIT])

cluster_upgrade_groups = list(cluster_create_groups)
cluster_upgrade_groups.extend([groups.CLUSTER_UPGRADE_WAIT])

cluster_config_groups = list(cluster_create_groups)
cluster_config_groups.extend([groups.CLUSTER_CFGGRP_DELETE])

cluster_config_actions_groups = list(cluster_config_groups)
cluster_config_actions_groups.extend([groups.CLUSTER_ACTIONS_CFGGRP_ACTIONS])

cluster_groups = list(cluster_actions_groups)
cluster_groups.extend([cluster_group.GROUP])

# Single-instance based groups
instance_create_groups = list(base_groups)
instance_create_groups.extend([groups.INST_CREATE,
                               groups.INST_DELETE_WAIT])

instance_error_create_groups = list(base_groups)
instance_error_create_groups.extend([instance_error_create_group.GROUP])

instance_force_delete_groups = list(base_groups)
instance_force_delete_groups.extend([instance_force_delete_group.GROUP])

instance_init_groups = list(base_groups)
instance_init_groups.extend([instance_create_group.GROUP,
                             instance_delete_group.GROUP])

instance_upgrade_groups = list(instance_create_groups)
instance_upgrade_groups.extend([instance_upgrade_group.GROUP])

backup_groups = list(instance_create_groups)
backup_groups.extend([groups.BACKUP,
                      groups.BACKUP_INST])

backup_incremental_groups = list(backup_groups)
backup_incremental_groups.extend([backup_group.GROUP])

backup_negative_groups = list(backup_groups)
backup_negative_groups.extend([groups.BACKUP_CREATE_NEGATIVE])

configuration_groups = list(instance_create_groups)
configuration_groups.extend([configuration_group.GROUP])

configuration_create_groups = list(base_groups)
configuration_create_groups.extend([groups.CFGGRP_CREATE,
                                    groups.CFGGRP_DELETE])

database_actions_groups = list(instance_create_groups)
database_actions_groups.extend([database_actions_group.GROUP])

guest_log_groups = list(instance_create_groups)
guest_log_groups.extend([guest_log_group.GROUP])

instance_actions_groups = list(instance_create_groups)
instance_actions_groups.extend([instance_actions_group.GROUP])

instance_groups = list(instance_actions_groups)
instance_groups.extend([instance_error_create_group.GROUP,
                        instance_force_delete_group.GROUP])

module_groups = list(instance_create_groups)
module_groups.extend([module_group.GROUP])

module_create_groups = list(base_groups)
module_create_groups.extend([groups.MODULE_CREATE,
                             groups.MODULE_DELETE])

replication_groups = list(instance_create_groups)
replication_groups.extend([groups.REPL_INST_DELETE_WAIT])

replication_promote_groups = list(replication_groups)
replication_promote_groups.extend([replication_group.GROUP])

root_actions_groups = list(instance_create_groups)
root_actions_groups.extend([root_actions_group.GROUP])

user_actions_groups = list(instance_create_groups)
user_actions_groups.extend([user_actions_group.GROUP])

# groups common to all datastores
common_groups = list(instance_create_groups)
# NOTE(lxkong): Remove the module related tests(module_groups) for now because
# of no use case.
common_groups.extend([guest_log_groups, instance_init_groups])

# Register: Component based groups
register(["backup"], backup_groups)
register(["backup_incremental"], backup_incremental_groups)
register(["backup_negative"], backup_negative_groups)
register(["cluster"], cluster_actions_groups)
register(["cluster_actions"], cluster_actions_groups)
register(["cluster_create"], cluster_create_groups)
register(["cluster_negative_actions"], cluster_negative_actions_groups)
register(["cluster_restart"], cluster_restart_groups)
register(["cluster_root"], cluster_root_groups)
register(["cluster_root_actions"], cluster_root_actions_groups)
register(["cluster_upgrade"], cluster_upgrade_groups)
register(["cluster_config"], cluster_config_groups)
register(["cluster_config_actions"], cluster_config_actions_groups)
register(["common"], common_groups)
register(["configuration"], configuration_groups)
register(["configuration_create"], configuration_create_groups)
register(["database"], database_actions_groups)
register(["guest_log"], guest_log_groups)
register(["instance"], instance_groups)
register(["instance_actions"], instance_actions_groups)
register(["instance_create"], instance_create_groups)
register(["instance_error"], instance_error_create_groups)
register(["instance_force_delete"], instance_force_delete_groups)
register(["instance_init"], instance_init_groups)
register(["instance_upgrade"], instance_upgrade_groups)
register(["module"], module_groups)
register(["module_create"], module_create_groups)
register(["replication"], replication_groups)
register(["replication_promote"], replication_promote_groups)
register(["root"], root_actions_groups)
register(["user"], user_actions_groups)

# Register: Datastore based groups
# These should contain all functionality currently supported by the datastore.
# Keeping them in alphabetical order may reduce the number of merge conflicts.
register(
    ["db2_supported"],
    single=[common_groups,
            configuration_groups,
            database_actions_groups,
            user_actions_groups, ],
    multi=[]
)

register(
    ["cassandra_supported"],
    single=[common_groups,
            backup_groups,
            database_actions_groups,
            configuration_groups,
            user_actions_groups, ],
    multi=[cluster_actions_groups,
           cluster_negative_actions_groups,
           cluster_root_actions_groups,
           cluster_config_actions_groups, ]
)

register(
    ["couchbase_supported"],
    single=[common_groups,
            backup_groups,
            root_actions_groups, ],
    multi=[]
)

register(
    ["couchdb_supported"],
    single=[common_groups,
            backup_groups,
            database_actions_groups,
            root_actions_groups,
            user_actions_groups, ],
    multi=[]
)

register(
    ["mariadb_supported"],
    single=[common_groups,
            backup_incremental_groups,
            configuration_groups,
            database_actions_groups,
            root_actions_groups,
            user_actions_groups, ],
    multi=[replication_promote_groups, ]
    # multi=[cluster_actions_groups,
    #        cluster_negative_actions_groups,
    #        cluster_root_actions_groups,
    #        replication_promote_groups, ]
)

register(
    ["mongodb_supported"],
    single=[common_groups,
            backup_groups,
            configuration_groups,
            database_actions_groups,
            root_actions_groups,
            user_actions_groups, ],
    multi=[cluster_actions_groups, ]
)

register(
    ["mysql_supported"],
    single=[common_groups,
            backup_incremental_groups,
            configuration_groups,
            database_actions_groups,
            instance_groups,
            instance_upgrade_groups,
            root_actions_groups,
            user_actions_groups, ],
    multi=[replication_promote_groups, ]
)

register(
    ["percona_supported"],
    single=[common_groups,
            backup_incremental_groups,
            configuration_groups,
            database_actions_groups,
            instance_upgrade_groups,
            root_actions_groups,
            user_actions_groups, ],
    multi=[replication_promote_groups, ]
)

register(
    ["postgresql_supported"],
    single=[common_groups,
            backup_incremental_groups,
            database_actions_groups,
            configuration_groups,
            root_actions_groups,
            user_actions_groups, ],
    multi=[replication_groups, ]
)

register(
    ["pxc_supported"],
    single=[common_groups,
            backup_incremental_groups,
            configuration_groups,
            database_actions_groups,
            root_actions_groups,
            user_actions_groups, ],
    multi=[]
    # multi=[cluster_actions_groups,
    #        cluster_negative_actions_groups,
    #        cluster_root_actions_groups, ]
)

# Redis instances does not support inherit root state from backups,
# so a customized root actions group is created, instance backuping
# and restoring tests will not be included.
redis_root_actions_groups = list(instance_create_groups)
redis_root_actions_groups.extend([groups.ROOT_ACTION_ENABLE,
                                  groups.ROOT_ACTION_DISABLE])
register(
    ["redis_supported"],
    single=[common_groups,
            backup_groups,
            configuration_groups,
            redis_root_actions_groups, ],
    multi=[replication_promote_groups, ]
    # multi=[cluster_actions_groups,
    #        cluster_negative_actions_groups,
    #        replication_promote_groups, ]
)

register(
    ["vertica_supported"],
    single=[common_groups,
            configuration_groups,
            root_actions_groups, ],
    multi=[cluster_actions_groups,
           cluster_negative_actions_groups,
           cluster_root_actions_groups, ]
)



# Copyright 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# based on configured release version
RPC_API_VERSION = "1.0"

# API version history:
#
# 1.0 - Initial version.  (We started keeping track at icehouse-3)
# 1.1 -
# 1.2 - ...
VERSION_ALIASES = {
    'icehouse': '1.0'
}
